import rasterio

def imgRead(imgpath,mode="r"):
    """
    Import satellite image to python env.

    Parameters
    ----------
    imgpath : Path to satellite image.
    mode : mode of the operation no need to change, optional
        Don't change. The default is "r".

    Returns
    -------
    imgf : Image as nd Numpy array.
    imgmeta : Metadata of the image.

    """
    with rasterio.open(imgpath,mode) as img:
        imgf=img.read()
        imgmeta=img.meta
    return imgf,imgmeta


def imgWrite(imgarray,filepath,imgmeta,mode="w",bands=1):
    """
    Export or write satellite image from python enviroment to disk.

    Parameters
    ----------
    imgarray : Array of the satellite image.
    filepath : output file destination.
        With directory if needed.
    imgmeta : metadata for the image 
        DESCRIPTION.
    mode : mode of the operation no need to change, optional
        Don't change. The default is "w".
    bands : Number of bands in the image, optional
        Change according to the bands. The default is 1.

    Returns
    -------
    Export image to disk.

    """
    if bands==1:
        imgmeta["count"]=bands
        with rasterio.open(filepath,mode,**imgmeta) as imgc:
            imgc.write(imgarray,1)
    else:
        imgmeta["count"]=bands
        with rasterio.open(filepath,mode,**imgmeta) as imgc:
            imgc.write(imgarray)




import os
import rasterio
from rasterio.warp import calculate_default_transform, reproject, Resampling

def reprojectR(imgpath,save_disk=False,out_proj=None,out_nodata=None,resample_method=Resampling.nearest):
    """
    Reproject raster from one coordinate system to another.

    Parameters
    ----------
    imgpath : Path to satellite image.
    save_disk : save image to disk, optional
        If true, it save reprojected image in the image directory. The default is False.
    out_proj : EPGS code in the format of 'EPGS:4326', to the coordinate system you want to reproject, optional
        visit https://epsg.io/ to get EPSG code. The default is None.
    out_nodata : nodata value for output image, optional
        It fill the outer values. The default is None.
    resample_method : Resampling method same as rasterio.warp.Resampling, optional
        Resampling.nearest, Resampling.bilinear, Resampling.cubic visit https://rasterio.readthedocs.io/en/latest/api/rasterio.warp.html for more info. The default is Resampling.nearest.

    Returns
    -------
    Image as ndArray
        Nd numpy array .
    Image metadata
        Metadata of the reprojected image.

    """
    if out_proj==None:
        return print("Provide a valid EPSG code in format of 'EPSG:4326'.")
    s=rasterio.open(imgpath)
    t, w, h = calculate_default_transform(s.crs, out_proj, s.width, s.height, *s.bounds)
    nmeta = s.meta.copy()
    nmeta.update({
        'nodata':out_nodata,
        'crs': out_proj,
        'transform': t,
        'width': w,
        'height': h
    })
    path=os.path.split(imgpath)[0]
    name="\\"+os.path.split(imgpath)[1].split(".")[0]+"_reproj"+"."+os.path.split(imgpath)[1].split(".")[1]
    if len(path)==0:
        path=os.getcwd()
    path+=name
    with rasterio.open(path, 'w', **nmeta) as d:
        for i in range(1, s.count + 1):
            reproject(
                source=rasterio.band(s, i),
                destination=rasterio.band(d, i),
                src_transform=s.transform,
                src_crs=s.crs,
                dst_transform=t,
                dst_crs=out_proj,
                dst_nodata=out_nodata,
                resampling=resample_method)

    with rasterio.open(path,"r") as re:
        imgA=re.read()
        meta=re.meta
    if save_disk==False:
        os.remove(path)
    return imgA , meta




import numpy
import pandas

def imagetoframe(imgarray):
    """
    Convert n-d image array to stucture data for ML algo.

    Parameters
    ----------
    imgarray : image as numpy ndarray.

    Returns
    -------
    satdf : satellite image as stuctured data.
        In the table columns represent bands and rows represent pixels.

    """
    imlist=[] # Blank list to store 1D array
    for i in range(imgarray.shape[0]):
        i2=imgarray[i].flatten() # Convert 2D array to 1D array
        imlist.append(i2) # Append to list
        imga=numpy.asarray(imlist) # Convert to array
        imgat=imga.transpose() # Transpose matrix for daataframe convertion
        satdf=pandas.DataFrame(imgat)
    return satdf

def frametoimage(imgframe,imgmeta):
    """
    To convert stuctured image frame to nd image array.

    Parameters
    ----------
    imgframe : Stuctured image data.
    imgmeta : Metadata of the image.
            Needed for get image width, height and band information.

    Returns
    -------
    rs : A nd numpy image array.

    """
    width=imgmeta["width"]
    height=imgmeta["height"]
    cou=len(imgframe.columns)
    narray=imgframe.to_numpy()
    if cou==1:
        tm=narray.transpose()
        rs=tm.reshape(height,width)
    else:
        tm=narray.transpose()
        iml=[]
        for i in range(tm.shape[0]):
            band=tm[i,:]
            band=band.reshape(height,width)
            iml.append(band)
        rs=numpy.asarray(iml)
    return rs
